/*
Benchmark for Route Planning Algorithms
 
Copyright (c) Microsoft Corporation

All rights reserved. 

MIT License
 
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#pragma once

#include <vector>

using namespace std;

namespace benchmark {

template<typename w_t>
class adjacency_array_graph {

public:

	// Exposed types.
	typedef int vertex_id_t;
	typedef int arc_id_t;
	typedef w_t weight_t;

	// This represents a vertex.
	struct vertex_t {
	public:
		arc_id_t first_arc_id;
		vertex_t() : first_arc_id(0) {}
	};

	// This represents a directed arc.
	struct arc_t {
	public:
		vertex_id_t to_vertex_id;
		weight_t weight;
		arc_t() : to_vertex_id(0), weight(0) {}	
	};
	
	
	// Number of vertices. The last vertex is a dummy vertex.
	inline size_t num_vertices() const { return vertices.size() - 1; }
	
	// Number of arcs.
	inline size_t num_arcs() const { return arcs.size(); }
	
	// Check consistency.
	// Returns true if the graph is consistent, otherwise false.
	inline bool check_consistency() const {
		// Check the vertices.
		if (vertices.size() == 0) return false;
		for (size_t v = 0; v < num_vertices(); ++v) {
			if (vertices[v].first_arc_id > vertices[v+1].first_arc_id) return false;
			if (vertices[v].first_arc_id < arc_id_t(0)) return false;
			if (vertices[v].first_arc_id > static_cast<arc_id_t>(arcs.size())) return false;
		}
		if (vertices[num_vertices()].first_arc_id != static_cast<arc_id_t>(arcs.size())) return false;
		
		// Check the arcs.
		for (size_t a = 0; a < num_arcs(); ++a) {
			if (arcs[a].to_vertex_id < 0) return false;
			if (arcs[a].to_vertex_id >= static_cast<vertex_id_t>(num_vertices())) return false;
		}
		
		return true;
	}

	// An array of vertices.
	// The last vertex is a dummy vertex pointing beyond the last arc id.
	vector<vertex_t> vertices;
	
	// An array of arcs.
	vector<arc_t> arcs;
};


// This is a default instantiation of the graph class.
typedef adjacency_array_graph<int> default_graph;

}