/*
Benchmark for Route Planning Algorithms
 
Copyright (c) Microsoft Corporation

All rights reserved. 

MIT License
 
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#pragma once

#include <fstream>
#include <iostream>
#include <string>
#include <cstdlib>
#include <vector>
#include <algorithm>

using namespace std;

#include "tools.h"

namespace benchmark {

// This is a small class of a sortable input arc.
template<typename vertex_id_t, typename weight_t>
struct gen_input_arc_t {
	vertex_id_t from_vertex_id;
	vertex_id_t to_vertex_id;
	weight_t weight;
	
	inline bool operator<(const gen_input_arc_t<vertex_id_t, weight_t> &other) const {
		if (from_vertex_id == other.from_vertex_id)
			return to_vertex_id < other.to_vertex_id;
		else
			return from_vertex_id < other.from_vertex_id;
	}
};


// Parses a Dimacs text file and produces an adjacency list graph.
// The return value indicates whether the parsing process succeeded.
template<typename graph_t>
bool parse_dimacs(const string filename, graph_t &graph) {
	typedef typename graph_t::vertex_id_t vertex_id_t;
	typedef typename graph_t::arc_id_t arc_id_t;
	typedef typename graph_t::weight_t weight_t;
	typedef gen_input_arc_t<vertex_id_t, weight_t> input_arc_t;
	cout << "Attempting to read Dimacs data from " << filename << "... " << flush;
	
	// Open file.
	ifstream file(filename.c_str());
	if (!file.is_open()) {
		cout << "could not open file. Did you call `make data' first?" << endl;
		return false;
	}
	
	cout << "opened." << endl;
	
	// Some counters and variables.
	vertex_id_t num_expected_vertices(0);
	arc_id_t num_expected_arcs(0);
	size_t num_arcs_parsed(0);
	vector<input_arc_t> input_arcs; // this will hold the input arcs (temporarily).
	
	// Clear stuff from graph.
	graph.vertices.clear();
	graph.arcs.clear();

	
	// Read the file line by line.
	string line("");
	while(!file.eof()) {
		getline(file, line);
		
		// Skip empty lines and those with a comment.
		if (line.empty()) continue;
		
		// Parse the header.
		if (line[0] == 'p') {
			const vector<string> tokens = split(line, ' ');
			num_expected_vertices = convert<vertex_id_t>(tokens[2]);
			num_expected_arcs = convert<arc_id_t>(tokens[3]);
			graph.vertices.reserve(num_expected_vertices + 1);
			
			cout << "Expecting " << num_expected_vertices << " vertices and " << num_expected_arcs << " directed arcs." << endl;
			cout << "Please wait, reading... " << flush;
		}
		
		// Parse an arc.
		if (line[0] == 'a') {
			const vector<string> tokens = split(line, ' ');
			input_arc_t input_arc;
			input_arc.from_vertex_id = convert<vertex_id_t>(tokens[1]) - 1; // dimacs vertex ids are 1-based.
			input_arc.to_vertex_id = convert<vertex_id_t>(tokens[2]) - 1; // dimacs vertex ids are 1-based.
			input_arc.weight = convert<weight_t>(tokens[3]);
			
			// Increase "degree" of the first vertex.
			if (graph.vertices.size() <= static_cast<size_t>(input_arc.from_vertex_id))
				graph.vertices.resize(input_arc.from_vertex_id + 1);
			++graph.vertices[input_arc.from_vertex_id].first_arc_id;
			
			input_arcs.push_back(input_arc);
			++num_arcs_parsed;
		}
		
	}
	
	file.close();
	cout << endl << "Finished reading the file (" << graph.vertices.size() << " vertices created, " << num_arcs_parsed << " arcs read)." << endl;
	
	// Add a dummy vertex to the end.
	graph.vertices.push_back(typename graph_t::vertex_t());
	
	
	// Sort the arcs by tail, then head.
	cout << "Sorting the arcs... " << flush;
	sort(input_arcs.begin(), input_arcs.end());
	cout << "done." << endl;
	
	// Build the adjacency list.
	cout << "Building adjacency array representation... " << flush;
	
	// First rewire the pointers.
	size_t index(0);
	for (size_t v_index(0); v_index < graph.vertices.size(); ++v_index) {
		const size_t next_index = index + graph.vertices[v_index].first_arc_id;
		graph.vertices[v_index].first_arc_id = index;
		index = next_index;
	}
	
	// Add arcs.
	graph.arcs.resize(input_arcs.size());
	for (size_t a_index(0); a_index < input_arcs.size(); ++a_index) {
		graph.arcs[a_index].to_vertex_id = input_arcs[a_index].to_vertex_id;
		graph.arcs[a_index].weight = input_arcs[a_index].weight;
	}
	
	cout << "done." << endl;
	
	// Clear temporary data structures.
	cout << "Clearing temporary data structures... " << flush;
	vector<input_arc_t>().swap(input_arcs);
	cout << "done." << endl;
	
	// Check the consistency of the graph.
	cout << "Checking consistency of the graph... " << flush;
	if (!graph.check_consistency()) {
		cout << "FAILED." << endl;
		return false;
	}
	cout << "OK." << endl;
	
	return true;
}


}